// -*- C++ -*-
//
//  This file is part of CBNSIM, the Content-Based Networking
//  Simulation Library.
//
//  Authors: See the file AUTHORS for full details.
//
//  Copyright (C) 2002-2004 University of Colorado
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
// $Id: SendMessageWNPF.cc,v 1.5 2004/12/13 17:50:56 carzanig Exp $
//
#include <string>
#include <cmath>
#include "cbnsim/SendMessageWNP.h"
#include "cbnsim/SendMessageWNPF.h"

#include <iostream>

using namespace cbnsim;
using namespace std;

SendMessageWNPF :: SendMessageWNPF()
    : WorkloadNodeProcessFactory( "send_message" ), m_delay( 0 ), m_const(false)
{
}

bool SendMessageWNPF :: applies_to( NodeType nt )
{
    // no need to allocate a process if the delay is zero
    return m_delay && nt == Publisher_Node;
}

bool SendMessageWNPF :: handle_argument( vector<string>::const_iterator& i,
					 vector<string>::const_iterator end ) throw ( argument_exception )
{
    if( (*i) == "-pt" )
    {
	if( ++i == end )
	{
	    throw argument_exception( "-pt argument expects a value" );
	}
	m_delay = atof( (*i).c_str() );
	return true;
    }
    else if( (*i) == "-p-const")
    {
	m_const = true;
	return true;
    }
    
    return false;
}

WorkloadNodeProcess* SendMessageWNPF :: newWorkloadNodeProcess( unsigned int nid,
								ostream& out, 
								ssbg::SSBG& ssbg,
								double wtu ) throw ()
{
    return new SendMessageWNP( nid, out, ssbg,
			       (unsigned int)floor( m_delay / wtu ), m_const );
}

void SendMessageWNPF :: print_optional_usage( std::ostream& out )
{
    out << "\t -pt        mean time to publish (in seconds)" << endl;
    out << "\t -p-const   hold publications at each node constant" << endl;
}

void SendMessageWNPF :: print_required_usage( std::ostream& out )
{
}

void SendMessageWNPF :: print_setting_summary( ostream& out )
{
    out << "# send_message: mean time to event = " << m_delay << " (sec)" << endl
	<< "# send_message: hold publications constant = " << m_const << endl;
}

bool SendMessageWNPF :: satisfied()  const
{
    return true; // no required settings
}
